﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Microsoft.Win32;
using System.IO;

namespace Schranka
{
    static class UserCommands
    {
        public static readonly RoutedCommand Original
            = new RoutedCommand
                  ("Original", typeof(UserCommands),
                   new InputGestureCollection() 
                   { new KeyGesture(Key.O, ModifierKeys.Alt) }
                  );

        public static readonly RoutedCommand Zvetsit
            = new RoutedCommand
                  ("Zvetsit", typeof(UserCommands),
                   new InputGestureCollection() 
                   { new KeyGesture(Key.Z, ModifierKeys.Alt) }
                  );
    }

    public partial class HlavniOkno : Window
    {
        private BitmapSource bms;
        private bool neprazdny, zvetseny;
        private string adresar, filtr;

        public HlavniOkno()
        {
            InitializeComponent();

            neprazdny = false;
            zvetseny = false;
            adresar = Directory.GetCurrentDirectory();
            filtr = "Soubory ve formátu JPG (*.jpg)|*.jpg|" + 
                    "Všechny soubory (*.*)|*.*";
        }

        public void UrciRozmery(out int sirka, out int vyska)
        {
            if (zvetseny)
            {
                sirka = 2 * bms.PixelWidth;
                vyska = 2 * bms.PixelHeight;
            }
            else
            {
                sirka = bms.PixelWidth;
                vyska = bms.PixelHeight;
            }
        }

        private void OpenCmd_LzeSpustit(object sender, CanExecuteRoutedEventArgs e)
        {
            e.CanExecute = true;
        }

        private void OpenCmd_Spusten(object sender, ExecutedRoutedEventArgs e)
        {
            OpenFileDialog odlgOtevrit = new OpenFileDialog();
            string jmSouboru;

            odlgOtevrit.InitialDirectory = adresar;
            odlgOtevrit.Filter = filtr;

            if (odlgOtevrit.ShowDialog()==true)
            {
                jmSouboru = odlgOtevrit.FileName;
                OtevriSoubor(jmSouboru);
            }
        }

        public void OtevriSoubor(string jmSouboru)
        {
            int sirka, vyska;

            try
            {
                bms = new BitmapImage(new Uri(jmSouboru));
                imgObrazek.Source = bms;

                UrciRozmery(out sirka, out vyska);
                grdGrid.ColumnDefinitions[0].Width = new GridLength(sirka);
                grdGrid.RowDefinitions[2].Height = new GridLength(vyska);
                neprazdny = true;
            }
            catch (NotSupportedException)
            {
                MessageBox.Show("Nepodporovaný formát obrázku.",
                    "Chyba", MessageBoxButton.OK,
                    MessageBoxImage.Error);
            }
        }

        private void SaveCmd_LzeSpustit(object sender, CanExecuteRoutedEventArgs e)
        {
            e.CanExecute = neprazdny;
        }

        private void SaveCmd_Spusten(object sender, ExecutedRoutedEventArgs e)
        {
            SaveFileDialog sdlgUlozit = new SaveFileDialog();
            JpegBitmapEncoder jEnc = new JpegBitmapEncoder();

            sdlgUlozit.InitialDirectory = adresar;
            sdlgUlozit.Filter = filtr;

            if (sdlgUlozit.ShowDialog()==true) 
            {
                jEnc.Frames.Add(BitmapFrame.Create(bms));
                using (FileStream fs = 
                    new FileStream(sdlgUlozit.FileName, FileMode.Create))
                {
                    jEnc.Save(fs);
                }
            }
        }

        private void CloseCmd_LzeSpustit(object sender, CanExecuteRoutedEventArgs e)
        {
            e.CanExecute = true;
        }

        private void CloseCmd_Spusten(object sender, ExecutedRoutedEventArgs e)
        {
            Close();
        }

        private void CopyCmd_LzeSpustit(object sender, CanExecuteRoutedEventArgs e)
        {
            e.CanExecute = neprazdny;
        }

        private void CopyCmd_Spusten(object sender, ExecutedRoutedEventArgs e)
        {
            Clipboard.SetImage(bms);
        }

        private void PasteCmd_LzeSpustit(object sender, CanExecuteRoutedEventArgs e)
        {
            e.CanExecute = Clipboard.ContainsImage();
        }

        private void PasteCmd_Spusten(object sender, RoutedEventArgs e)
        {
            BitmapSource image = Clipboard.GetImage();

            /*
            bms = new FormatConvertedBitmap
                  (image, PixelFormats.Bgr24, null, 0);
            */
            int sirka, vyska;

            
            using (MemoryStream ms = new MemoryStream())
            {
                BitmapEncoder enc = new BmpBitmapEncoder();
                enc.Frames.Add(BitmapFrame.Create(image));
                enc.Save(ms);

                ms.Seek(0, SeekOrigin.Begin);
                bms = BitmapFrame.Create
                    (ms, BitmapCreateOptions.IgnoreImageCache, BitmapCacheOption.OnLoad);
            }

            imgObrazek.Source = bms;

            UrciRozmery(out sirka, out vyska);

            grdGrid.ColumnDefinitions[0].Width = new GridLength(sirka);
            grdGrid.RowDefinitions[2].Height = new GridLength(vyska);
            neprazdny = true;
        }

        private void OriginalCmd_LzeSpustit(object sender, CanExecuteRoutedEventArgs e)
        {
            e.CanExecute = zvetseny;
        }

        private void OriginalCmd_Spusten(object sender, RoutedEventArgs e)
        {
            grdGrid.ColumnDefinitions[0].Width = new GridLength(bms.PixelWidth);
            grdGrid.RowDefinitions[2].Height = new GridLength(bms.PixelHeight);
            zvetseny = false;
        }

        private void ZvetsitCmd_LzeSpustit(object sender, CanExecuteRoutedEventArgs e)
        {
            e.CanExecute = !zvetseny && neprazdny;
        }

        private void ZvetsitCmd_Spusten(object sender, RoutedEventArgs e)
        {
            grdGrid.ColumnDefinitions[0].Width = new GridLength(2 * bms.PixelWidth);
            grdGrid.RowDefinitions[2].Height = new GridLength(2 * bms.PixelHeight);
            zvetseny = true;
        }

        private void prvek_IsEnabledChanged(object sender, DependencyPropertyChangedEventArgs e)
        {
            Image prvek=(Image)sender;

            if (prvek.IsEnabled == true)
                prvek.Opacity = 1;
            else
                prvek.Opacity = 0.3;
        }

        private void wndHlavniOkno_Loaded(object sender, RoutedEventArgs e)
        {
            if (Environment.GetCommandLineArgs().Length==2)
                OtevriSoubor(Environment.GetCommandLineArgs()[1]);
        }

        private void wndHlavniOkno_Drop(object sender, DragEventArgs e)
        {
            string[] soubory;

            if (e.Data.GetDataPresent(DataFormats.FileDrop))
            {
                soubory = (string[])e.Data.GetData(DataFormats.FileDrop);
                OtevriSoubor(soubory[0]);
            }
        }
    }
}
